/*
## Copyright (C) 2006   Torsten Finke   <fi@igh-essen.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Revision: 1.5 $
$Date: 2006/06/11 02:27:52 $
$RCSfile: prime-gen.cc,v $

The code below actually is only a wrapper around Daniel Bernsteins Primegen
library functions (version 0.97, see http://cr.yp.to/primegen.html). 

*/

#include <octave/oct.h>

#include "prime-gen.c"

DEFUN_DLD (primegen, args, ,
	   "-*- texinfo -*-\n\
@deftypefn{Loadable Function} {@var{p} =} primegen([@var{a},] @var{b})\n\
\n\
@cindex prime list\n\
\n\
Calculate list of primes between non-negative scalar arguments @var{a} and @var{b}\n\
using Bernsteins Primegen library. @var{a} defaults to zero. \n\
\n\
Example:\n\
@example \n\
primegen(5, 19)\n\
@end example\n\
@noindent\n\
returns\n\
@example\n\
ans =\n\
   5   7  11  13  17  19  \n\
@end example\n\
\n\
Reference:\n\
http://cr.yp.to/primegen.html\n\
\n\
@end deftypefn\n\
@seealso{list_primes, primes, primecnt}\n\
")
{    
    
#   define myself "primegen"
    
    RowVector p; 
    octave_value r; 
    primegen pg;
    int nargin = args.length ();
    idlong a, b; 
    udlong i, n, ua, ub; 
    
    if ( nargin < 1 || ! args(0).is_numeric_type() ) {
	error(myself " expects at least one numeric argument"); 
        return r;
    }
    if ( nargin > 1 ) {
	if ( ! args(1).is_numeric_type() ) {
	    error(myself " expects only numeric arguments"); 
	    return r;
	} 
	a = args(0).int_value(); 
	b = args(1).int_value(); 
    } else {
	a = 0; 
	b = args(0).int_value(); 
    }
    b ++; 
    if ( a < 0 ) {
	error(myself " is only defined for intervals starting at non-negative bounds"); 
	return r;
    }
    if ( b < 2 ) {
	error(myself " is only defined for arguments greater than one"); 
	return r;
    }
    
    ua = (udlong) a; 
    ub = (udlong) b; 
    primegen_init(&pg);
    primegen_skipto(&pg, ua);
    n = primegen_count(&pg, ub); 
    
    p = RowVector(n); 
    
    primegen_init(&pg);
    primegen_skipto(&pg, ua);
    for (i = 0 ; i < n; i++) {
	p(i) = primegen_next(&pg);
    }
    
    return octave_value(p); 
}

DEFUN_DLD (primecnt, args, ,
	   "-*- texinfo -*-\n\
@deftypefn{Loadable Function} {@var{p} =} primecnt(@var{n})\n\
\n\
@cindex prime list\n\
\n\
Calculate number of primes up to non-negative matrix arguments @var{n}\n\
using Bernsteins Primegen library. \n\
\n\
Example:\n\
@example \n\
primecnt([10, 20; 13, -2])\n\
@end example\n\
@noindent\n\
returns\n\
@example\n\
ans =\n\
     4     8\n\
     6   NaN\n\
@end example\n\
\n\
Reference:\n\
http://cr.yp.to/primegen.html\n\
\n\
@end deftypefn\n\
@seealso{list_primes, primes, primegen}\n\
")
{    
    
#   define myname "primecnt"
    
    Matrix p; 
    octave_value r; 
    primegen pg;
    int nargin = args.length ();
    udlong i, k, pr, pc;
    
    if ( nargin < 1 || ! args(0).is_numeric_type() ) {
	error(myname " expects at least one numeric argument"); 
        return r;
    }
    
    p = args(0).matrix_value(); 
    pr = p.rows();
    pc = p.columns();
    
    for (i = 0; i < pr; i++) {
	for (k = 0; k < pc; k++) {
	    if (p(i, k) > 1) {
		primegen_init(&pg);
		p(i, k) = primegen_count(&pg, (udlong) (p(i, k) + 1)); 
	    } else {
		p(i, k) = 0; 
	    }
	}
    }
    
    return octave_value(p); 
}


